/* ------------------------------------------------ */
vm= 'Monitor for APC Back-UPS CS/ES (USB) v 0.04b1'
/* ------------------------------------------------ */
/*                        16.06.2006 (c) Larin Yury */

/* set constants */
Interval         = 15           /* sleep interval, sec. */
IntervalDIV      = 3            /* sleep interval divider (if power fail) */
ShutDownInterval = 300          /* shutdown interval after power fail, sec. */
ShutDownBC       = 15           /* minimal battery capacity, % */
BTimeMin         = 90           /* minimal battery lifetime, sec. */

ShtDwn           = 'shutdown o' /* ShutDown command */
ddName           = 'UPS$'       /* set device driver name */
ProductName      = 'Back-UPS'   /* set valid product name */

icon_position    = '94,90'      /* icon position on the desktop */

/* --- Fix for holded USB. Set '0' if USB-port working ok. --- */
UseFix           = 1
MinPower         = 1250         /* minimal PowerSummary, CentiVolts */

/* ----------------------------------------- */
if RxFuncQuery('SysLoadFuncs') then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
 end
/* ----------------------------------------- */
call SysCls
say vm
say
say '(Press <1> to close monitoring)'
say

install_dir = directory()          /* APCUSBM's home dir */

/* ensure that the device driver statement exists in config.sys */
if stream(ddName,'command','query exists') \= '\DEV\' || ddName
then do
  say 'Missing DEVICE=x:\OS2\BOOT\USBECD.SYS /N:'||ddName 'statement in CONFIG.SYS.'
  say
  say 'Press any key to close...'
  Key = SysGetKey('NOECHO')
  exit
  end

rc=stream(ddName,'command','open')
oiBuffer = ReadUPS(x2c('80 06 00 01 00 00 12'), 26)
say 'Found on device driver' ddName':'
say 'Manufacturer             -- 'disp(15)
Product = disp(16)
say 'Product                  -- 'Product
if substr(Product,1,8) \= ProductName then do
  say
  say 'Unknown product name.'
  say
  say 'Press any key to close...'
  Key = SysGetKey('NOECHO')
  exit
 end

rc=SysDestroyObject("<UPS_STAT>");
rc=SysCreateObject("WPProgram","UPS Status","<WP_DESKTOP>",,
"OBJECTID=<UPS_STAT>;EXENAME="install_dir"\status.cmd;STARTUPDIR="install_dir";ICONFILE="install_dir"\Unknown.ico;"||,
"ICONPOS="icon_position";","UPDATE");

oiBuffer = ReadUPS(x2c('a1 01 0c 03 00 00 07'), 15)
rc=stream(ddName,'command','close')

BC = c2d(substr(oiBuffer,2,1))
if (BC <= 100)|(c2d(substr(oiBuffer,2,1)) = 12) then say 'Current battery capacity -- 'BC'%'
 else say 'Error reading battery capacity.'

BTime = c2d(substr(oiBuffer,6,1)||substr(oiBuffer,5,1))
if ShutDownInterval > BTime then ShutDownInterval = BTime

say
say 'Log:'
cnt = 0
PG = 2

/* ----------------------------------------- */
do forever
 rc=stream(ddName,'command','open')
 if rc = 'READY:' then do
   PS = UPSStatus(x2c('a1 01 13 03 00 00 07'),2)
   
   /* --- Fix for holded USB --- */
   if ((PS = 1300)|(PS = 1301))&(UseFix \= 0) then
    if PG = 2 then PSold = PS
     else if PSold \= PS then UseFix = 0
      else do
        PSold = PS
        PSF = UPSStatus(x2c('a1 01 09 03 00 00 07'),3)
        if left(PSF,2) = '09' then do
          if MinPower <= x2d(right(PSF, 2)||substr(PSF, 3, 2)) then PS = 1301
           else PS = 1300
         end
        else say date() time() '-- Error reading data!!! (PSF = 'PSF')'
       end
   /* ------- End of fix ------- */

   select
    when PS = 1301 then if PG \= 1 then do
       say date() time() '-- AC utility power source good'
       rc=SysCreateObject("WPProgram","AC Power good","<WP_DESKTOP>",,
          "OBJECTID=<UPS_STAT>;ICONFILE="install_dir"\ACGood.ico;","UPDATE");
       rc=SysSetObjectData("<UPS_STAT>","TITLE=AC Power good");
       if PG = 0 then Interval = Interval * IntervalDIV
       PG = 1
       cnt = 0
      end
    when PS = 1300 then do
      if PG \= 0 then do
        say date() time() '-- AC utility power source fails!!!'
        parse value SysCurPos() with Crow Ccol
        Interval = Interval / IntervalDIV
        PG = 0
        rc=SysCreateObject("WPProgram","AC Power fail!","<WP_DESKTOP>",,
           "OBJECTID=<UPS_STAT>;ICONFILE="install_dir"\ACFail.ico;","UPDATE");
        rc=SysSetObjectData("<UPS_STAT>","TITLE=AC Power fail");
       end

      BC1 = UPSStatus(x2c('a1 01 0c 03 00 00 07'),6)
      LTime = ShutDownInterval - cnt
      if substr(BC1,1,2) = '0C' then do
         BC = x2d(substr(BC1,3,2))
         BTime = x2d(right(BC1,2)||substr(BC1,9,2))
         if BTime < LTime then LTime = BTime
        end
       else do
         BC = 101
         BTime = BTimeMin
        end

      SF1 = UPSStatus(x2c('a1 01 06 03  00 00 07'),4)
      if substr(SF1,1,2) = '06' then SF = substr(SF1,7,2)
       else SF = 49

      parse value SysCurPos(Crow,0) with Crow1 Ccol
      if BC <= 100 then say time() 'Battery capacity 'BC'%; Estimate time for shutdowning 'LTime' sec.        '
       else say time() 'Error reading battery capacity. Estimate time for shutdowning 'LTime' sec.'

      if (BC <= ShutDownBC + 10)|(BTime < BTimeMin + 90) then do
        rc=SysCreateObject("WPProgram","Battery LOW!","<WP_DESKTOP>",,
           "OBJECTID=<UPS_STAT>;ICONFILE="install_dir"\BLow.ico;","UPDATE");
        rc=SysSetObjectData("<UPS_STAT>","TITLE=Battery LOW!");
       end

      if (cnt >= ShutDownInterval)|(BC <= ShutDownBC)|(BTime < BTimeMin)|(SF = 50) then do
        rc=stream(ddName,'command','close')
        say date() time() '-- Sysyem shutdouning...'
        say '*** cnt= 'cnt'; BC= 'BC'; BTime= 'BTime'; SF='SF
        rc=SysDestroyObject("<UPS_STAT>")
        ShtDwn
        exit
       end

      cnt = cnt + Interval
     end
    when PS = 0 then do
      say date() time() '-- Error reading from device driver' ddName'.'
      rc=SysCreateObject("WPProgram","UPS Error!","<WP_DESKTOP>",,
         "OBJECTID=<UPS_STAT>;ICONFILE="install_dir"\Error.ico;","UPDATE");
      rc=SysSetObjectData("<UPS_STAT>","TITLE=UPS Error!");
      PG = 2
     end
    otherwise say date() time() '-- Error reading data!!! (PS = 'PS')'
   end
  end
  else do
    say date() time() '-- Device driver' ddName 'currently in use.'
    PG = 2
   end

 rc=stream(ddName,'command','close')

 if chars()=1 then do
   Key = SysGetKey('NOECHO')
   If Key = 1 Then do
     say date() time() '-- Monitor is closed by console'
     rc=SysDestroyObject("<UPS_STAT>")
     exit
    end
   say 'Pressed <'||key||'>. To close monitor press <1>.'
  end

 call SysSleep Interval
end

/* ----------------------------------------- */
UPSStatus:
 oiBuffer = left(arg(1), 15, x2c(00))
 rc=charout(ddName,oiBuffer)
 rc=stream(ddName,'description')
 if rc \= 'READY:' then return 0
return c2x(substr(oiBuffer,9,arg(2)))

ReadUPS:
 oiBuffer1 = left(arg(1), arg(2), x2c(00))
 rc=charout(ddName,oiBuffer1)
 rc=stream(ddName,'description')
 if rc \= 'READY:' then do
  parse value rc with sState ':' mNumber
  say SysGetMessage(mNumber,,ddName)
  say
  say 'Press any key to close...'
  Key = SysGetKey('NOECHO')
  exit
 end
return substr(oiBuffer1,9,arg(2)-9)

disp:
 ixString = substr(oiBuffer, arg(1), 1)
 if c2d(ixString) = 0 then return 'No String!'

 siBuffer = left(x2c('80 06 00 03 09 04 01'),9,x2c(00))
 siBuffer = overlay(ixString,siBuffer,3)
 rc=charout(ddName,siBuffer)
 rc=stream(ddName,'description')
 if rc \= 'READY:' then do
   /* obtain and issue error message */
   parse value rc with sState ':' mNumber
   if mNumber == 95 /* character i/o call interrupted */
   then RETURN "I/O canceled (4 seconds time-out) ..."
   else RETURN 'String Error! ('rc')'
  end

 szBuffer = substr(siBuffer,9,1)
 if c2d(szBuffer) = 0 then return 'No String!'

 siBuffer = overlay(szBuffer,siBuffer,7,c2d(szBuffer)+2,x2c(00))
 rc=charout(ddName,siBuffer)
 rc=stream(ddName,'description')
 if rc \= 'READY:' then return 'String Error!'

 szString=c2d(substr(siBuffer,9,1)) + 8
 usString = ''
 do xPos = 11 to szString by 2
  usString = usString || substr(siBuffer,xPos,1)
 end
return usString
